# noccylabs/gd-image: OO API for GD

This library provides an object-oriented API around the GD image functions. If
you are looking for a high-level library to do fancy shit, you are probably
looking for `noccylabs/gd-canvas`.

All image operations are self-modifying unless otherwise specified.

### Drawing pixels and shapes


### Drawing images

You can draw an image onto an image like this:

```php
// Load the images
$image = Image::fromFile("image.png");
$overlay = Image::fromFile("overlay.png");

// Create a new rect from the dimensions of $overlay and move it to sit in
// the bottom right corner of $image's rect.
$destRect = Rect::fromImage($overlay);
$destRect->moveTo($image->getWidth() - $overlay->getWidth() - 5, 
                  $image->getHeight() - $overlay->getHeight() - 5);
// Draw $overlay into $image
$image->drawImage($overlay, $destRect, CopyMergeMixer::class);

```


### Drawing text

GdImage abstracts away the difference between bitmap fonts and TrueType fonts.

```php
// Load built-in font 0-5
$font = new BitmapFont(0);
// Or load a bitmap font from file
$font = new BitmapFont(__DIR__."/font.gdf");

// Draw text on the image 
$image->drawText(5, 5, $font, "Hello World", 0x000000);
```

This means that the font size had to be a property of the font instance, considering
that `BitmapFont` is not sizeable.

```php
// Load a TrueType font
$font = new TrueTypeFont(__DIR__."/font.ttf", 24);

// Draw text on the image 
$image->drawText(5, 5, $font, "Hello World", 0x000000);
```



## Components

### Convolution

### Process

The `ProcessInterface` encapsulates all operations that take place on an 
image, such as GDs built-in filters.

### Transformation

The `TransformationInterface` encapsulates all operations that perform a 
transformation on an image, such as an affine transformatin.

### Mixer



## Dev Guidelines

* The various helpers should always operate on `\GdImage` and not on the
  custom `NoccyLabs\GdImage\Image`. This lets the components be reused
  for other code without fully implementing the library. The fonts can
  thus for example be used directly on any `\GdImage`.