<?php

namespace NoccyLabs\GdImage\Convolution;
use InvalidArgumentException;

class Convolution implements ConvolutionInterface
{

    protected array $matrix;

    protected float $divisor;

    protected float $offset;

    public function __construct(array $matrix, ?float $divisor=null, float $offset=0)
    {
        if (count($matrix) !== 9) {
            throw new InvalidArgumentException("The convolution matrix should be 9 values (3x3)");
        }
        
        if ($divisor === null) {
            $divisor = array_sum($matrix);
        }

        $this->offset = $offset;
        $this->divisor = $divisor;
        $this->matrix = array_chunk($matrix, 3, false);
    }

    
	/**
	 * Return a 3x3 matrix used in the imageaffine call
	 * @return array
	 */
	public function getMatrix(): array
    {
        return $this->matrix;
	}
	
	/**
	 * Return the
	 * @return float
	 */
	public function getDivisor(): float
    {
        return $this->divisor;
	}
	
	/**
	 * Return the offset or bias to apply to the pixels
	 * @return float
	 */
	public function getOffset(): float
    {
        return $this->offset;
	}
}

