<?php

namespace NoccyLabs\GdImage\Font;
use GdFont;
use GdImage;
use NoccyLabs\GdImage\Image;


class BitmapFont implements FontInterface
{
    const OPT_ROTATE = 'rotate';
    const OPT_SHADOW = 'shadow';
    const OPT_OUTLINE = 'outline';

    private GdFont|int $font = 1;

    public int $lineSpacing = 0;

    public function __construct(GdFont|string|int $font)
    {
        if (is_string($font)) {
            $font = imageloadfont($font);
            if ($font === false)
                throw new \InvalidArgumentException("Invalid .gdf font file: {$font}");
        }

        $this->font = $font;
    }

    public function measureString(string $text, array $options): array
    {
        $lines = explode("\n", $text);
        $length = max(array_map("strlen", $lines));
        $lines = count($lines);

        $fw = imagefontwidth($this->font) * $length;
        $fh = imagefontheight($this->font) * $lines + $this->lineSpacing * ($lines - 1);

        $rotate = intval($options[self::OPT_ROTATE] ?? 0) % 360;

        $offs = (object) ['l' => 0, 't' => 0, 'r' => 0, 'b' => 0];
        if ($options[self::OPT_OUTLINE]??null) {
            $offs->l = 1;
            $offs->t = 1;
            $offs->r = 1;
            $offs->b = 1;
        } elseif ($options[self::OPT_SHADOW]??null) {
            $offs->r = 1;
            $offs->b = 1;
        }

        if ($rotate == 0) {
            return [ $fw + $offs->r + $offs->l, $fh + $offs->b + $offs->t, -$offs->l, -$offs->t ];
        } elseif ($rotate == 270) {
            return [ $fh + $offs->r + $offs->l, $fw + $offs->b + $offs->t, -$offs->l, -$offs->t ];
        }

        throw new \RuntimeException("Bad rotation for BitmapFont (only 0, 270)");
    }

    public function renderString(GdImage $image, int $x, int $y, int $color, string $text, array $options)
    {
        $outline = $options[self::OPT_OUTLINE] ?? null;
        $shadow = $options[self::OPT_SHADOW] ?? null;

        $lines = explode("\n", $text);
        $height = imagefontheight($this->font);
        for ($line = 0; $line < count($lines); $line++) {
            $liney = $y + $line * ($height + $this->lineSpacing);
            if ($outline !== null) {
                imagestring($image, $this->font, $x - 1, $liney - 1, $lines[$line], $outline);
                imagestring($image, $this->font, $x - 1, $liney + 0, $lines[$line], $outline);
                imagestring($image, $this->font, $x - 1, $liney + 1, $lines[$line], $outline);
                imagestring($image, $this->font, $x + 0, $liney - 1, $lines[$line], $outline);
                imagestring($image, $this->font, $x + 0, $liney + 1, $lines[$line], $outline);
                imagestring($image, $this->font, $x + 1, $liney - 1, $lines[$line], $outline);
                imagestring($image, $this->font, $x + 1, $liney + 0, $lines[$line], $outline);
                imagestring($image, $this->font, $x + 1, $liney + 1, $lines[$line], $outline);
            }
            if ($shadow !== null) {
                imagestring($image, $this->font, $x + 1, $liney + 1, $lines[$line], $shadow);
            }
            imagestring($image, $this->font, $x, $liney, $lines[$line], $color);
        }
    }
}