<?php

namespace NoccyLabs\GdImage\Font;
use GdFont;
use GdImage;
use NoccyLabs\GdImage\Image;


class TrueTypeFont implements FontInterface
{

    private string $filename;

    private int $size;

    public function __construct(string $filename, int $size)
    {
        $this->filename = $filename;
        $this->size = $size;
    }

    public function measureString(string $text, array $options): array
    {
        $angle = intval($options['angle'] ?? 0);
        $boundingBox = imagettfbbox($this->size, $angle, $this->filename, $text);

        [$x1,$y1,$x2,$y2,$x3,$y3,$x4,$y4] = $boundingBox;
        $width = $x2 - $x1;
        $height = abs($y3 - $y2);
        $left = $x1;
        $top = $y3 - $y2;

        // TODO calculate box
        return [ $width, $height, $left, $top ];
    }

    public function renderString(GdImage $image, int $x, int $y, int $color, string $text, array $options)
    {
        //printf("renderstring x=%d y=%d c=%06x s='%s'\n", $x, $y, $color, $text);

        $angle = intval($options['angle'] ?? 0);
        $boundingBox = imagettfbbox($this->size, $angle, $this->filename, $text);

        [$x1,$y1,$x2,$y2,$x3,$y3,$x4,$y4] = $boundingBox;

        $x = $x + $x1;
        $y = $y - $y3;

        imagettftext($image, $this->size, $angle, $x, $y, $color, $this->filename, $text);

    }


    public static function setFontSearchPath(string $path)
    {
        putenv("GDFONTPATH=" . $path);
    }
}