<?php

namespace NoccyLabs\GdImage;

class Gradient
{

    private array $stops = [];

    public function __construct(...$stops)
    {
        $parsedStops = [];
        $lastMeasured = null;
        $current = 0;

        foreach ($stops as $stop) {
            [$x,$c] = is_array($stop)?$stop:[null,$stop];
            $c = new Color($c);
            if ($current == 0) {
                $x = 0;
                $lastMeasured = 0;
            } else {
                if ($current == count($stops) - 1) {
                    $x = 1.0;
                }
                if ($x !== null) {
                    $lastStop = $parsedStops[$lastMeasured][0];
                    $numStops = $current - $lastMeasured;
                    if ($numStops > 1) {
                        $step = ($x - $lastStop) / $numStops;
                        for ($s = $lastMeasured + 1; $s < $current; $s++) {
                            $parsedStops[$s][0] = $parsedStops[$s-1][0] + $step;
                        }
                    }
                    $lastMeasured = $current;
                }
            }
            $parsedStops[$current++] = [$x,$c];
        }
        
        $this->stops = $parsedStops;

    }

    public function __invoke(float $x): Color
    {
        $stopStart = null;
        //printf("get=%.2f\n", $x);
        foreach ($this->stops as $n=>[$a,$c]) {
            //printf("  stop=%.2f\n", $a);
            if ($a <= $x) {
                $stopStart = $n;
            } else {
                break;
            }
        }

        [$r1,$g1,$b1] = $this->stops[$stopStart][1]->getComponents();
        [$r2,$g2,$b2] = $this->stops[$stopStart+1][1]->getComponents();
        $x1 = $this->stops[$stopStart][0];
        $x2 = $this->stops[$stopStart+1][0];

        $pos = 1.0 - (($x - $x1) * (1.0 / ($x2 - $x1)));

        $r = ($r1 * $pos) + ($r2 * (1.0 - $pos));
        $g = ($g1 * $pos) + ($g2 * (1.0 - $pos));
        $b = ($b1 * $pos) + ($b2 * (1.0 - $pos));

        return new Color($r,$g,$b);

    }

}
