<?php

namespace NoccyLabs\GdImage\Mixer;

class CopyMixerTest extends \PhpUnit\Framework\TestCase
{

    /**
     * @covers NoccyLabs\GdImage\Mixer\CopyMixer::blend
     */
    public function testMixingWithAlphaBlending()
    {

        $im1 = imagecreatetruecolor(64,64);
        imagealphablending($im1, false);
        for ($x = 0; $x < 64; $x++) {
            imageline($im1, $x, 0, $x, 63, (0x7F - ($x * 2)) << 24 | 0x0000FF);
        }
        //imagesavealpha($im1, true);
        //imagepng($im1, "/tmp/alpha1.png");

        $im2 = imagecreatetruecolor(64, 64);
        imagefilledrectangle($im2, 0, 0, 63, 63, 0xFF0000);
        //imagepng($im2, "/tmp/alpha2.png");

        $mixer = new CopyMixer(true);
        $mixer->blend($im1, $im2, 64, 64);
        //imagepng($im2, "/tmp/alpha3.png");

        $expect = "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAIAAAAlC+aJAAAACXBIWXMAAA7EAAAOxAGVKw4bAAAAY0lEQVRoge3PAQnAMBDAwO+Yf8MrrDKOQo4YyPpnvnn3tT1zuQa0BrQGtAa0BrQGtAa0BrQGtAa0BrQGtAa0BrQGtAa0BrQGtAa0BrQGtAa0BrQGtAa0BrQGtAa0BrQGtAa0A7IJQH4adQZHAAAAAElFTkSuQmCC";
        $this->assertImageEquals($expect, $im2);

    }

    /**
     * @covers NoccyLabs\GdImage\Mixer\CopyMixer::blend
     */
    public function testMixingWithoutAlphaBlending()
    {

        $im1 = imagecreatetruecolor(64,64);
        imagealphablending($im1, false);
        for ($x = 0; $x < 64; $x++) {
            imageline($im1, $x, 0, $x, 63, (0x7F - ($x * 2)) << 24 | 0x0000FF);
        }
        //imagesavealpha($im1, true);
        //imagepng($im1, "/tmp/alpha1.png");

        $im2 = imagecreatetruecolor(64, 64);
        imagefilledrectangle($im2, 0, 0, 63, 63, 0xFF0000);
        //imagepng($im2, "/tmp/alpha2.png");

        $mixer = new CopyMixer(false);
        $mixer->blend($im1, $im2, 64, 64);
        //imagepng($im2, "/tmp/alpha3.png");

        $expect = "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAIAAAAlC+aJAAAACXBIWXMAAA7EAAAOxAGVKw4bAAAAYElEQVRoge3PQQ0AIBDAsAP/nkEEj4ZkVbCtmTM/2zrgVQNaA1oDWgNaA1oDWgNaA1oDWgNaA1oDWgNaA1oDWgNaA1oDWgNaA1oDWgNaA1oDWgNaA1oDWgNaA1oDWgPaBVCHAX8eIsxwAAAAAElFTkSuQmCC";
        $this->assertImageEquals($expect, $im2);

    }

    protected function assertImageEquals(string $expectBase64, \GdImage $image)
    {
        ob_start();
        imagepng($image, null);
        $raw = base64_encode(ob_get_contents());
        ob_end_clean();

        $this->assertEquals($expectBase64, $raw);
    }

}